/*--------------------------------------------------------------------
  frequency1: An audio frequency meter using Timer1 counter.
			  Handles 1 Hz to 1 MHz with readout on LCD display.
 
  Processor: PIC16F684
  
  Discussion:
    Timer1 is used as a 16 bit counter with external clock, the
    clock being the input signal. By turning on the timer for
    exactly 1 second, the frequency can be directly taken from 
    the timer register word. That is, until the count exceeds 64K,
    at which point the timer counter overflows. The overflows are 
    counted in the timer1 interrupt handler, which becomes the  
    high-order word of the count. So we can count pretty high as
    far as software is concerned. The count is limited to
    9,999,999 in the ltoa module because the actual high bound is
    around 1 MHz and we want to use the smallest display possible.
--------------------------------------------------------------------*/                
#include <system.h>
#include "frequency1.h"

#pragma CLOCK_FREQ      4000000

// Config Bits: 11 0000 1100 0100
// bit 13-12: unimplemented, set to 1
// bit 11:    FCMEN  0 = fail safe clock disabled
// bit 10:    IESO   0 = internal external switchover disabled
// bits 9-8:  BODEN  00 = Brown out detect disabled
// bit 7:     !CPD   1 = data memory code protection disabled
// bit 6:     !CP    1 = program memory code protection disabled
// bit 5:     MCLRE  0 = RA3/!MCLR pin function is RA3
// bit 4:     ^PWRTE 0 = PWRT enabled
// bit 3:     WDTE   0 = watchdog timer disabled
// bits 2-0:  FOSC   100 = INTOSC oscillator, I/O on RA4/OSC2/CLKOUT, I/O on RA5/OSC1/CLKIN

#pragma DATA _CONFIG, 0x30c4

// LCD settings (see lcd_driver.h)
#define LCD_ARGS 	1,	/* Interface type: mode 0 = 8bit, 1 = 4bit(low nibble), 2 = 4bit(upper nibble) */ \
		0, 				/* Use busy signal: 1 = use busy, 0 = use time delays */\
		PORTC, TRISC, 	/* Data port and data port tris register */ \
		PORTC, TRISC, 	/* Control port and control port tris register */ \
		5,				/* Bit number of control port is connected to RS */ \
		6,				/* Bit number of control port is connected to RW */ \
		4 				/* Bit number of control port is connected to Enable */

// Note that RW is not connected to the processor in this project.
// But the 16F684 doesn't have a RC6 port either!

#include <lcd_driver.h> // include the LCD template code

#define RA4             0x0004		// diagnostic LED

// Global Variables

char writeDelayType=0;		// used by LCD routines

static unsigned short usOverflowCtr;

// Functions

static void initCPU( void );

/*---------------------------------------------------------------------
  main()
---------------------------------------------------------------------*/
void main()
{
  unsigned long  ulCtr;
  unsigned short usTimer1_word;
  char buffer[8];
  
  initCPU();                	// initialize hardware
  lcd_setup();					// initialize lcd
  lcd_datamode();
  usOverflowCtr = 0;

  while ( 1 )
  {
    tmr1l = tmr1h = 0;
    usOverflowCtr = 0;
    set_bit( t1con, TMR1ON );		// start timer1 counter

    delay_ms( 250 );				// wait 1 second
    delay_ms( 250 );
    delay_ms( 250 );
    delay_ms( 250 );
    
    clear_bit( t1con, TMR1ON );		// stop timer1 counter
    
    MAKESHORT( usTimer1_word, tmr1l, tmr1h );
    ulCtr = ( (unsigned long) usOverflowCtr << 16 ) | (unsigned long) usTimer1_word;
    lcd_clear();
    
    ltoa( ulCtr, buffer );
    lprintf( buffer );   
  }
}

/*---------------------------------------------------------------------
  interrupt()
  
  Handles the timer2 interrupt.
---------------------------------------------------------------------*/
void interrupt( void )
{
  static unsigned char t2ctr=0;
  
  if ( test_bit( pir1, TMR1IF ) )
  {
    // The timer1 interrupt occurs when Timer1, operating as a 
    // counter, rolls over.
    ++usOverflowCtr;
    clear_bit( pir1, TMR1IF );		// reset timer1 overflow interrupt flag
    return;
  }

#if 0    
  if ( test_bit( pir1, TMR2IF ) )
  {
    // The timer2 interrupt occurs ( 256 * prescale * postscale * Fosc/4 ).
    // When Fosc/4 is 1 microsecond, and pre and postscale are 16,
    // the interrupt occurs every 65.5 milliseconds.
    
    ++t2ctr;
    if ( t2ctr == 8 )
      set_bit( porta, RA4 );		// turn LED on
    else
      if ( t2ctr == 16 )
      {
        clear_bit( porta, RA4 );
        t2ctr = 0;
      }
    
    clear_bit( pir1, TMR2IF );		// reset timer2 overflow interrupt flag
  }
#endif
}

/*---------------------------------------------------------------------
  initCPU()
---------------------------------------------------------------------*/
static void initCPU()
{
  // Initialize the internal clock:
  osccon = 01100000b;		// internal clock @ 4MHz, use FOSC (config)

  // PORTC connections as follows:
  //  RC0 - (out) LCD DB4
  //  RC1 - (out) LCD DB5
  //  RC2 - (out) LCD DB6
  //  RC3 - (out) LCD DB7
  //  RC4 - (out) LCD E
  //  RC5 - (out) LCD R/S

  portc = 0;
  cmcon0 = 00000111b;			// comparators off
  ansel = 0;					// digital I/O
  trisc = 00000000b;            // 1=input, 0=output

  // PORTA connections as follows:
  //  RA0/AN0/ICSPDAT - (in) (used for programming only)
  //  RA1/AN1/ICSPCLK - (in) (used for programming only)
  //  RA2/AN2/T0CKI - (in) signal input to timer0 counter
  //  RA3/!MCLR/Vpp - (in) (used for programming only)
  //  RA4/AN3 - (out) - diagnostic LED
  //  RA5/T1CKI - (in) signal input to timer1 counter

  porta = 00000000b;			// make RA4 low, LED off
  trisa = 00101111b;			// 1=input, 0=output

  // Timer1 initialization. Use Timer1 as 16-bit counter with
  // input on RA5/T1CKI.

  tmr1l = tmr1h = 0;
  t1con = 00000110b;      	// 1:1 prescale; LP osc off; no sync; ext. clock; stopped
  set_bit( pie1, TMR1IE );      // Timer1 interrupts

  // Timer2 initialization. Timer2 is used to generate interrupts
  // to turn an LED on and off, to show the program is running.
  
  //t2con = 01111111b;			// 1:16 postscale; timer2 on; 1:16 prescale
  //set_bit( pie1, TMR2IE );      // Timer2 interrupts

  // Enable interrupts for Timers

  set_bit( intcon, GIE );       // enable global interrupts
  set_bit( intcon, PEIE );      // enable peripheral interrupts
}
